clear; clc;

% Predator Ecology book, Chapter 9
% Code to make Figure 9.4
% Functional responses for copepod Paracartia grani foraging on
% Isochrysis galbana. Data from Helenius and Saiz 2017

[num,words] = xlsread('Helenius_and_Saiz_2017.xlsx','Sheet1'); % read in data

num_offered = num(:,20)./100; % pull all of the prey levels data
num_eaten = num(:,21)./100; % /100 to scale down to 1/100th of a mL

% sort data by number offered for later bootstrapping
sorted = sortrows([num_offered,num_eaten],1);
num_offered = sorted(:,1);
num_eaten = sorted(:,2);

exp_duration = 24; % specify how long experiment ran

%% do a quick type II fit to get guesses on parameters

% Here we will use LambertW version of the Roger's Random Predator Equation
typeII = fittype( ['x - lambertw(a*h*x*exp(-a*(',num2str(exp_duration),' - h*x)))/(a*h)'], 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( typeII );
opts.StartPoint = [0.1 1/max(num_offered)];
opts.Lower = [0 0];
opts.Upper = [1 10];

[fit_result, fit_gof] = fit(num_offered,num_eaten,typeII,opts); % do the fit for small prey
    S_coeffs = coeffvalues(fit_result); % output the parameters
    a_S = S_coeffs(1);
    h_S = S_coeffs(2);

% predict prey eaten at each prey level offered (for guess below)
x = unique(num_offered);
predicted_curve = x-lambertw(a_S.*h_S.*x.*exp(-a_S.*(exp_duration-h_S.*x)))./(a_S.*h_S);

% quick visualizatio of data and initial fit
figure(2);clf(2);
box on; hold on;
plot(num_offered,num_eaten,'ok');
plot(x,predicted_curve,'-k');
shg;

%% we need an initial log-likelihood for our OLS fit for comparison with asymptotic a

test_parameters = [a_S,h_S];
prey_to_use = unique(num_offered); % identify tested prey levels in data
LL = []; % open an empty matrix to store log-likelihood values
for ii = 1:length(prey_to_use) % run through prey level
    test_prey = prey_to_use(ii); % pull out each prey level in sequence
    % use numerical solver to predict Re given parameters and Ro
    [predicted_rate, fval] = fsolve(@(z)rrp_type_II(z,[test_parameters,exp_duration,test_prey]), 0.5*test_prey);
    data = num_eaten(num_offered == test_prey); % pull out data for that prey level
    LL(length(LL)+1:length(LL)+length(data)) = -log(normpdf(data,predicted_rate)); % calculate pdf for the estimated mean given the data
end
LL(find(LL==Inf)) = max(LL(find(LL~=Inf)));
sum_LL_typeII = sum(LL); % sum up all the - log-likelihoods
AIC_typeII = -2*sum_LL_typeII + 2*2;

%% now estimate parameters using maximum likelihood for the aymptotic a model
num_attemps = 500; % how many parameter sets to test per bootstrapped dataset

%%% step 1, open up a first loop for bootstrapping
num_bootstraps = 30;
for i = 1:num_bootstraps
    
    % create a bootstrapped data set, stratified across prey levels
    num_offered_BS = [];
    num_eaten_BS = [];
    for jj = 1:length(x) % cycle through the prey levels
        how_many = length(num_offered(num_offered==x(jj))); % how many reps at prey level jj
        index = randsample(how_many,how_many,'true'); % which elements to draw
        % stitch together each stratified sample into one vector
        num_offered_BS(length(num_offered_BS)+1:length(num_offered_BS)+how_many) = num_offered(length(num_eaten_BS)+index);
        num_eaten_BS(length(num_eaten_BS)+1:length(num_eaten_BS)+how_many) = num_eaten(length(num_eaten_BS)+index);
    end
    
    %%% step 2, open up a loop for MCMC parameter searching 
    xm = [2*a_S h_S 200 sum_LL_typeII]; % set guesses, LL starts at Inf
    log_z = []; % open empty vector
    rng('shuffle');
    
    for k = 1:num_attemps
        % randomly draw a displacement from current best estimate
        a = xm(1) + 0.2*randn*xm(1);
            while a < 0 || a > 1
                a = xm(1) + 0.2*randn*xm(1);
            end
        h = xm(2) + 0.1*randn*xm(2);
            while h < 0
                h = xm(2) + 0.1*randn*xm(2);
            end
        kr = xm(3) + 0.2*randn*xm(3);
            while kr < 0 || kr > 0.5*max(num_offered)
                kr = xm(3) + 0.2*randn*xm(3);
            end
        
        test_parameters = [a h kr]; % compile test parameters to pass out
        
        %%% step 3, calculate log-likelihood for the test parameters
        prey_to_use = unique(num_offered); % identify tested prey levels in data
        LL = []; % open an empty matrix to store log-likelihood values
        for ii = 1:length(prey_to_use) % run through prey level
            test_prey = prey_to_use(ii); % pull out each prey level in sequence
            % use numerical solver to predict Re given parameters and Ro
            % use the OLS type II fit as a guess for Re
            [predicted_rate, fval] = fsolve(@(z)asym_a_type_III(z,[test_parameters,exp_duration,test_prey]), 0.5*test_prey);
            data = num_eaten(num_offered == test_prey); % pull out data for that prey level
            LL(length(LL)+1:length(LL)+length(data)) = -log(normpdf(data,predicted_rate)); % calculate pdf for the estimated mean given the data
        end
        LL(find(LL==Inf)) = max(LL(find(LL~=Inf)));
        sum_LL = sum(LL); % sum up all the - log-likelihoods
        
        if sum_LL < xm(4) && sum_LL > -Inf
            xm = [a h kr sum_LL];
            log_z(size(log_z,1)+1,1) = sum_LL;
            log_z(size(log_z,1),2) = kr;        
        end
    end
    log_xm(i,:) = xm; % log the final parameters and LL
end

%% collect median values and cis

a_median = median(log_xm(:,1));
h_median = median(log_xm(:,2));
kr_median = median(log_xm(:,3));

% cis on estimated k
ci_k_up = prctile(log_xm(:,3),97.5);
ci_k_down = prctile(log_xm(:,3),2.5);
% cis on estimated asym_a
ci_a_up = prctile(log_xm(:,1),97.5);
ci_a_down = prctile(log_xm(:,1),2.5);

%% we need a log-likelihood for our asymptotic a fit's median parameters

test_parameters = [a_median,h_median];
prey_to_use = unique(num_offered); % identify tested prey levels in data
LL = []; % open an empty matrix to store log-likelihood values
for ii = 1:length(prey_to_use) % run through prey level
    test_prey = prey_to_use(ii); % pull out each prey level in sequence
    % use numerical solver to predict Re given parameters and Ro
    [predicted_rate, fval] = fsolve(@(z)rrp_type_II(z,[test_parameters,exp_duration,test_prey]), 0.5*test_prey);
    data = num_eaten(num_offered == test_prey); % pull out data for that prey level
    LL(length(LL)+1:length(LL)+length(data)) = -log(normpdf(data,predicted_rate)); % calculate pdf for the estimated mean given the data
end
LL(find(LL==Inf)) = max(LL(find(LL~=Inf)));
sum_LL_asym_a = sum(LL); % sum up all the - log-likelihoods
AIC_asym_a = -2*sum_LL_asym_a + 2*2;
deltaAIC = AIC_typeII-AIC_asym_a

%% solve for curves

% range of x's over which to solve
x = 1:2:max(num_offered);
% get the original OLS type II curve
typeII_curve = x-lambertw(a_S.*h_S.*x.*exp(-a_S.*(72-h_S.*x)))./(a_S.*h_S);

% solve for predicted number eaten for all estimates
for jj = 1:size(log_xm,1) % loop over the bootstrapped data sets
    for kk = 1:length(x) % loop over the prey levels
    [asym_a_curve(jj,kk), fval] = fsolve(@(z)asym_a_type_III(z,[log_xm(jj,1),log_xm(jj,2),log_xm(jj,3),exp_duration,x(kk)]), 0.25*x(kk));
    end
end

% solve for predicted number eaten for median estimate
for kk = 1:length(x) % loop over the prey levels
    [asym_a_mean(kk), fval] = fsolve(@(z)asym_a_type_III(z,[a_median,h_median,kr_median,exp_duration,x(kk)]), 0.5*x(kk));
end

% solve for prey eaten at Kr
[f_at_k, fval] = fsolve(@(z)asym_a_type_III(z,[a_median,h_median,kr_median,exp_duration,kr_median]), 10);
    
%% plot results

figure(1); clf(1);
subplot(2,1,1); % plot functional response
    hold on; box on;

    h1 = plot(x,asym_a_curve','-','Color',[0.8 0.8 0.8]);
    h3 = plot([0,x],[0,asym_a_mean],'-k','LineWidth',2);
    h2 = plot([0,x],[0,typeII_curve],'--k','LineWidth',2);       
    h4 = plot([kr_median kr_median],[0 f_at_k],':k','LineWidth',2);
    h1 = plot(num_offered,num_eaten,'ok');
    legend([h1 h3 h2 h4],'Data','Asymptotic a','Type II','K_{R}','Location','Best');
    xlabel('Number prey offered');
    ylabel('Number prey eaten');

subplot(2,1,2); % plot estimated space clearance rate and K_R
    hold on; box on;

    h1 = plot(log_xm(:,1),log_xm(:,3),'ok','MarkerFaceColor',[0.8 0.8 0.8]);
    h2 = plot(a_S,0,'pk','MarkerFaceColor','w');
    h4 = plot([ci_a_down ci_a_up],[kr_median kr_median],'-k');
         plot([a_median a_median],[ci_k_down ci_k_up],'-k');
    h3 = plot(a_median,kr_median,'ok','MarkerFaceColor','w');        
    legend([h1 h2 h3 h4],'ML estimates','OLS type II','Mean ML','CIs','Location','Best');
    xlabel('Space clearance rate');
    ylabel('Half saturation prey level');
    
gtext('A','FontSize',12);
gtext('B','FontSize',12);