clear; clc;

% Predator Ecology book, Chapter 10
% Code to make Figure 10.3
% Recreating the functional responses for Norops serranoi lizards foraging on
% Mexican fruit flies (Anastrepha ludens) using simulations that include
% stochasticity and individual variation in parameters

[num,words] = xlsread('Dor_et_al.xlsx','Sheet1'); % read in original data

num_offered = num(:,20);
num_eaten = num(:,21);
plot_location = words(:,13);

% We'll just do males here because they are more clearly type II
num_offered_M = num_offered(ismember(plot_location,'1a'));
num_eaten_M = num_eaten(ismember(plot_location,'1a'));

% Redo the original fit to get paramaters
% Here we will use LambertW version of the Roger's Random Predator Equation
% Duration of the experiment was 6 hours
ft = fittype( 'x - lambertw(a*h*x*exp(-a*(6 - h*x)))/(a*h)', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( ft );
opts.Display = 'Off';
opts.StartPoint = [0.8 0.05];
opts.Lower = [0 0];
opts.Upper = [1 2];

[M_result, M_gof] = fit(num_offered_M,num_eaten_M,ft,opts); % do the fit for males
    M_coeffs = coeffvalues(M_result); % output the parameters
    a_M = M_coeffs(1);
    h_M = M_coeffs(2);
    M_rsquare = M_gof.rsquare;
    x = 0:1:70; % a sequence of x's to calculate over
    M_curve = x-lambertw(a_M.*h_M.*x.*exp(-a_M.*(6-h_M.*x)))./(a_M.*h_M);

%% simulate the data set WITH individual variation

% make space clearance a lognormal distsribution with mean a_M and variance V
std_a = 1.2.*a_M;
V_a = std_a.^2;
MU_a = log(a_M.^2 ./ sqrt(V_a+a_M.^2));
SIGMA_a = sqrt(log(V_a./a_M.^2 + 1));

% make handling time a lognormal distsribution with mean h_M and variance V
std_h = 1.2.*h_M;
V_h = std_h.^2;
MU_h = log(h_M.^2 ./ sqrt(V_h+h_M.^2));
SIGMA_h = sqrt(log(V_h./h_M.^2 + 1));

num_simulations = 10; % pick number of simulations to run
y0 = [2 6 10 20 30 40 50 60]; % starting number of prey in trial
t_max = [1:0.25:8,1:0.25:8,1:0.25:8]; % time frame (hours of experiment)
prey_offered = y0.*ones(num_simulations,length(y0));

for k = 1:length(t_max)
    for j = 1:length(y0) % across prey levels
        for i = 1:num_simulations % reps at prey level
            t = 0;
            prey_level = y0(j);
            a_M_R = lognrnd(MU_a,SIGMA_a); % here now pull a random space clearance rate
            h_M_R = lognrnd(MU_h,SIGMA_h); % here now pull a random handling time
                while t(end) < t_max(k)
                    if prey_level(end) > 0
                        current_rate = a_M_R*prey_level(end);
                        time_to_kill = exprnd(1/current_rate);
                        t(end+1) = t(end) + time_to_kill + h_M_R;
                        prey_level(end+1) = prey_level(end) - 1;
                    else
                        break
                    end
                end
                within_time = t<t_max(k);
                last_predation = find(within_time,1,'last');
                prey_eaten(i,j) = y0(j) - prey_level(last_predation);
        end
    end

        num_trials_all_eaten(k) = length(find(prey_eaten == prey_offered))

        ft = fittype( ['x - lambertw(a*h*x*exp(-a*(',num2str(t_max(k)),' - h*x)))/(a*h)'], 'independent', 'x', 'dependent', 'y' );
            opts = fitoptions( ft );
            opts.Display = 'Off';
            opts.StartPoint = [0.8 0.05];
            opts.Lower = [0 0];
            opts.Upper = [1 2];

        x = reshape(y0.*ones(1,num_simulations)',1,numel(prey_offered))';
        y = reshape(prey_eaten,1,numel(prey_offered))';
        [M_result, M_gof] = fit(x,y,ft,opts); % do the fit for males
        M_coeffs = coeffvalues(M_result); % output the parameters
        a_M_record(k) = M_coeffs(1);
end

%% now refit
figure(1);clf(1);
subplot(1,2,1);
plot(t_max,num_trials_all_eaten,'ok');
subplot(1,2,2);
plot(num_trials_all_eaten,a_M_record,'ok'); lsline
shg;

% 
% 
% %% now plot the second simulated data set
% subplot(1,2,2);
%     hold on; box on;
%     plot(num_offered_M,num_eaten_M,'ok','MarkerFaceColor','w');
%     plot(y0.*ones(1,num_simulations)',prey_eaten','ok','MarkerFaceColor','k');
%     plot(x,M_curve,'-k');
%     title('With individual variation');
%     ylim([0 50]);
%     xlim([0 70]);
%     xlabel('Prey offered','FontSize',12);
%     ylabel('Prey consumed','FontSize',12);
%     
% shg;
% 
% gtext('A','FontSize',12);
% gtext('B','FontSize',12);
