clear; clc;

% Predator Ecology book, Chapter 10
% Code to make Figure 10.3
% Recreating the functional responses for Norops serranoi lizards foraging on
% Mexican fruit flies (Anastrepha ludens) using simulations that include
% stochasticity and individual variation in parameters
[num,words] = xlsread('BD norops serranoi.xlsx','Males'); % read in data
num_offered_M = num(:,6);
num_eaten_M = num(:,8);

% Redo the original fit to get paramaters
% Here we will use LambertW version of the Roger's Random Predator Equation
% Duration of the experiment was 6 hours
ft = fittype( 'x - lambertw(a*h*x*exp(-a*(6 - h*x)))/(a*h)', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( ft );
opts.Display = 'Off';
opts.StartPoint = [0.2 0.2];
opts.Lower = [0 0];

[M_result, M_gof] = fit(num_offered_M,num_eaten_M,ft,opts); % do the fit for males
    M_coeffs = coeffvalues(M_result); % output the parameters
    a_M = M_coeffs(1);
    h_M = M_coeffs(2);
    M_rsquare = M_gof.rsquare;
    x = 0:1:70; % a sequence of x's to calculate over
    M_curve = x-lambertw(a_M.*h_M.*x.*exp(-a_M.*(6-h_M.*x)))./(a_M.*h_M);

%% simulate the data set - first time with no individual variation

num_simulations = 10; % pick number of simulations to run
y0 = [2 6 10 20 30 40 50 60]; % starting number of prey in trial
t_max = 6; % time frame (hours of experiment)

for j = 1:length(y0)
    for i = 1:num_simulations
        t = 0;
        prey_level = y0(j);
            while t(end) < t_max
                if prey_level(end) > 0
                    current_rate = a_M*prey_level(end);
                    time_to_kill = exprnd(1/current_rate);
                    t(end+1) = t(end) + time_to_kill + h_M;
                    prey_level(end+1) = prey_level(end) - 1;
                else
                    break
                end
            end
            within_time = t<6;
            last_predation = find(within_time,1,'last');
            prey_eaten(j,i) = y0(j) - prey_level(last_predation);
    end
end

%% plot first simulated data set, no individual variation
figure(1); clf(1);
subplot(2,1,1);
    hold on; box on;
    z1 = plot(num_offered_M,num_eaten_M,'ok','MarkerFaceColor','w');
    z2 = plot(y0.*ones(1,num_simulations)',prey_eaten','ok','MarkerFaceColor','k');
    z3 = plot(x,M_curve,'-k');
    title('Without individual variation');
    legend([z1 z2(1) z3],'Observed data','Simulated','Original fit','Location','Best');
    ylim([0 50]);
    xlim([0 70]);
    xlabel('Prey offered','FontSize',12);
    ylabel('Prey consumed','FontSize',12);

%% simulate the data set WITH individual variation

% make space clearance a lognormal distsribution with mean a_M and variance V
std_a = 1.2*a_M;
V_a = std_a^2;
MU_a = log(a_M^2 / sqrt(V_a+a_M^2));
SIGMA_a = sqrt(log(V_a/a_M^2 + 1));

% make handling time a lognormal distsribution with mean h_M and variance V
std_h = 1.2*h_M;
V_h = std_h^2;
MU_h = log(h_M^2 / sqrt(V_h+h_M^2));
SIGMA_h = sqrt(log(V_h/h_M^2 + 1));

for j = 1:length(y0)
    for i = 1:num_simulations
        t = 0;
        prey_level = y0(j);
        a_M_R(j,i) = lognrnd(MU_a,SIGMA_a); % here now pull a random space clearance rate
        h_M_R = lognrnd(MU_h,SIGMA_h); % here now pull a random handling time
            while t(end) < t_max
                if prey_level(end) > 0
                    current_rate = a_M_R(j,i)*prey_level(end);
                    time_to_kill = exprnd(1/current_rate);
                    t(end+1) = t(end) + time_to_kill + h_M_R;
                    prey_level(end+1) = prey_level(end) - 1;
                else
                    break
                end
            end
            within_time = t<6;
            last_predation = find(within_time,1,'last');
            prey_eaten(j,i) = y0(j) - prey_level(last_predation);
    end
end

%% now plot the second simulated data set
subplot(2,1,2);
    hold on; box on;
    sim_offered = y0.*ones(1,num_simulations)';
    plot(num_offered_M,num_eaten_M,'ok','MarkerFaceColor','w');
    plot(sim_offered,prey_eaten','ok','MarkerFaceColor','k');
    plot(x,M_curve,'-k');
    title('With individual variation');
    ylim([0 50]);
    xlim([0 70]);
    xlabel('Prey offered','FontSize',12);
    ylabel('Prey consumed','FontSize',12);
    
shg;

gtext('A','FontSize',12);
gtext('B','FontSize',12);

%%

offered_reshaped = reshape(sim_offered',[numel(sim_offered) 1]);
eaten_reshaped = reshape(prey_eaten,[numel(prey_eaten) 1]);

[M_result2, M_gof2] = fit(offered_reshaped,eaten_reshaped,ft,opts); % do the fit for males
    M_coeffs2 = coeffvalues(M_result2); % output the parameters
    a_M2 = M_coeffs2(1);
    h_M2 = M_coeffs2(2);
    
ft = fittype( 'log(x - lambertw(a*h*x*exp(-a*(6 - h*x)))/(a*h))', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( ft );
opts.Display = 'Off';
opts.StartPoint = [0.2 0.2];
opts.Lower = [0 0];

x = offered_reshaped(eaten_reshaped>0)
y = eaten_reshaped(eaten_reshaped>0)
[M_result2, M_gof2] = fit(x,log(y),ft,opts); % do the fit for males
    M_coeffs2 = coeffvalues(M_result2); % output the parameters
    a_M2 = M_coeffs2(1);
    h_M2 = M_coeffs2(2);
    
ft = fittype( '(1/(x - lambertw(a*h*x*exp(-a*(6 - h*x)))/(a*h)))', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( ft );
opts.Display = 'Off';
opts.StartPoint = [0.2 0.2];
opts.Lower = [0 0];

[M_result2, M_gof2] = fit(x,log(1./y),ft,opts); % do the fit for males
    M_coeffs2 = coeffvalues(M_result2); % output the parameters
    a_M2 = M_coeffs2(1);
    h_M2 = M_coeffs2(2);
    
    